# -*- coding: utf-8 -*-

"""
cuadraai

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from cuadraai.api_helper import APIHelper
from cuadraai.configuration import Server
from cuadraai.http.api_response import ApiResponse
from cuadraai.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from cuadraai.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from cuadraai.models.paginated_response_ex_list_model_ex import PaginatedResponseExListModelEx
from cuadraai.models.model_ex import ModelEx
from cuadraai.exceptions.error_response_exception import ErrorResponseException


class ModelsController(BaseController):

    """A Controller to access Endpoints in the cuadraai API."""
    def __init__(self, config):
        super(ModelsController, self).__init__(config)

    def get_models(self,
                   page=None,
                   size=None):
        """Does a GET request to /model.

        This endpoint display all of our AI models.

        Args:
            page (int, optional): The request query parameter.
            size (int, optional): The request query parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. Chat
                successfully processed.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/model')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('page')
                         .value(page))
            .query_param(Parameter()
                         .key('size')
                         .value(size))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('OAuth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaginatedResponseExListModelEx.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad request, read again our documentation or contact support for guidance.', ErrorResponseException)
            .local_error('401', 'Not authorized, check our OAuth2 doc.', ErrorResponseException)
            .local_error('500', 'Internal error, if this error persist, please contact support.', ErrorResponseException)
        ).execute()

    def create_model(self,
                     body):
        """Does a POST request to /model.

        This endpoint creates a new custom Model for you to train and use.

        Args:
            body (ModelEx): The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. Chat
                successfully processed.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/model')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body)
                        .is_required(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('OAuth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ModelEx.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad request, read again our documentation or contact support for guidance.', ErrorResponseException)
            .local_error('401', 'Not authorized, check our OAuth2 doc.', ErrorResponseException)
            .local_error('500', 'Internal error, if this error persist, please contact support.', ErrorResponseException)
        ).execute()

    def get_model(self,
                  id):
        """Does a GET request to /model/{id}.

        This endpoint shows you information about a particular model given an
        id.

        Args:
            id (str): The request template parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. Chat
                successfully processed.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/model/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .is_required(True)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('OAuth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ModelEx.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad request, read again our documentation or contact support for guidance.', ErrorResponseException)
            .local_error('401', 'Not authorized, check our OAuth2 doc.', ErrorResponseException)
            .local_error('500', 'Internal error, if this error persist, please contact support.', ErrorResponseException)
        ).execute()

    def remove_model(self,
                     id):
        """Does a DELETE request to /model/{id}.

        This endpoint removes a custom model you created.

        Args:
            id (str): The request template parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. Chat
                successfully processed.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/model/{id}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .is_required(True)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('OAuth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ModelEx.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad request, read again our documentation or contact support for guidance.', ErrorResponseException)
            .local_error('401', 'Not authorized, check our OAuth2 doc.', ErrorResponseException)
            .local_error('500', 'Internal error, if this error persist, please contact support.', ErrorResponseException)
        ).execute()

    def update_model(self,
                     id,
                     body):
        """Does a PATCH request to /model/{id}.

        This endpoint updates a custom model you created.

        Args:
            id (str): The request template parameter.
            body (ModelEx): The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. Chat
                successfully processed.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/model/{id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .is_required(True)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body)
                        .is_required(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('OAuth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ModelEx.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad request, read again our documentation or contact support for guidance.', ErrorResponseException)
            .local_error('401', 'Not authorized, check our OAuth2 doc.', ErrorResponseException)
            .local_error('500', 'Internal error, if this error persist, please contact support.', ErrorResponseException)
        ).execute()
